@@ -18,17 +18,18 @@ from functools import reduce, partial
 
 from typing import Dict, Callable, List, Tuple, Union, Optional
 
-from collections import namedtuple
 import math
+import enum
 from operator import mul
 
 import numpy as onp
 
-from jax import lax, ops, vmap, eval_shape
+from jax import lax, ops, vmap, eval_shape, tree_map
 from jax.abstract_arrays import ShapedArray
 from jax.interpreters import partial_eval as pe
 import jax.numpy as jnp
 
+from jax_md import dataclasses
 from jax_md import quantity
 from jax_md import space
 from jax_md import util
@@ -39,6 +40,8 @@ high_precision_sum = util.high_precision_sum
 # Typing
 
 Array = util.Array
+PyTree = util.PyTree
+
 f32 = util.f32
 f64 = util.f64
 
@@ -47,6 +50,48 @@ i64 = util.i64
 
 DisplacementOrMetricFn = space.DisplacementOrMetricFn
 
+# Parameter Trees
+
+
+class ParameterTreeMapping(enum.Enum):
+  """An enum specifying how parameters are processed in mapped functions.
+
+  Attributes:
+    Global: Global parameters are passed directly to the mapped function.
+    PerParticle: PerParticle parameters are combined in pairs based on the
+      particle index. E.g. `p_ij = combinator(p_i, p_j)` for particles i and j.
+      These parameters are expected to have a leading axis of length the number
+      of particles.
+    PerBond: PerBond parameters are expected to have leading two dimensions
+      equal to the number of particles in the system.
+    PerSpecies: PerSpecies parameters are expected to have two leading
+      dimensions equal to the number of species. For particles of species `s_i`
+      and `s_j` parameters are combined according to
+      `p_ij = combinator(p[s_i], p[s_j])`.
+  """
+  Global = 0
+  PerParticle = 1
+  PerBond = 2
+  PerSpecies = 3
+
+
+@dataclasses.dataclass
+class ParameterTree:
+  """A container denoting that parameters are in the form of a PyTree.
+
+  Attributes:
+    tree: A JAX PyTree containing a tree of parameters. Before being fed into
+      mapped functions, these parameters are processed according to the
+      mapping.
+    mapping: A ParameterTreeMapping object that specifies how the parameters
+      are processed. 
+  """
+  tree: PyTree
+  mapping: ParameterTreeMapping = dataclasses.static_field()
+
+
+Parameter = Union[ParameterTree, Array, float]
+
 
 # Mapping potential functional forms to bonds.
 
@@ -65,6 +110,14 @@ def _get_bond_type_parameters(params: Array, bond_type: Array) -> Array:
     else:
       raise ValueError(
           'Params must be a scalar or a 1d array if using a bond-type lookup.')
+  elif isinstance(params, ParameterTree):
+    if params.mapping is ParameterTreeMapping.Global:
+      return params.tree
+    elif params.mapping is ParameterTreeMapping.PerBond:
+      return tree_map(lambda p: p[bond_type], params.tree)
+    else:
+      raise ValueError('ParameterTreeMapping must be either Global or PerBond'
+                       'if used with `smap.bond`.')
   elif(isinstance(params, int) or
        isinstance(params, float) or
        jnp.issubdtype(params, jnp.integer) or
@@ -96,19 +149,19 @@ def bond(fn: Callable[..., Array],
 
   Args:
     fn: A function that takes an ndarray of pairwise distances or displacements
-      of shape `[n, m]` or `[n, m, d_in]` respectively as well as kwargs specifying
-      parameters for the function. `fn` returns an ndarray of evaluations of shape
-      `[n, m, d_out]`.
+      of shape `[n, m]` or `[n, m, d_in]` respectively as well as kwargs
+      specifying parameters for the function. `fn` returns an ndarray of
+      evaluations of shape `[n, m, d_out]`.
     metric: A function that takes two ndarray of positions of shape
       `[spatial_dimension]` and `[spatial_dimension]` respectively and returns
       an ndarray of distances or displacements of shape `[]` or `[d_in]`
       respectively. The metric can optionally take a floating point time as a
       third argument.
-    static_bonds: An ndarray of integer pairs wth shape `[b, 2]` where each pair
-      specifies a bond. `static_bonds` are baked into the returned compute
+    static_bonds: An ndarray of integer pairs wth shape `[b, 2]` where each
+      pair specifies a bond. `static_bonds` are baked into the returned compute
       function statically and cannot be changed after the fact.
-    static_bond_types: An ndarray of integers of shape `[b]` specifying the type
-      of each bond. Only specify bond types if you want to specify bond
+    static_bond_types: An ndarray of integers of shape `[b]` specifying the
+      type of each bond. Only specify bond types if you want to specify bond
       parameters by type. One can also specify constant or per-bond parameters
       (see below).
     ignore_unused_parameters: A boolean that denotes whether dynamically
@@ -119,13 +172,15 @@ def bond(fn: Callable[..., Array],
       where no bond type information is provided these should be either
 
         1. a scalar
-        2. an ndarray of shape `[b]`. 
-      
-      If bond type information is provided then the parameters should be specified as either
+        2. an ndarray of shape `[b]`.
+
+      If bond type information is provided then the parameters should be
+      specified as either
 
         1. a scalar
         2. an ndarray of shape `[max_bond_type]`.
-
+        3. a ParameterTree containing a PyTree of parameters and a mapping. See
+           ParameterTree for details.
   Returns:
     A function `fn_mapped`. Note that `fn_mapped` can take arguments bonds and
     `bond_types` which will be bonds that are specified dynamically. This will
@@ -172,7 +227,8 @@ def bond(fn: Callable[..., Array],
 # Mapping potential functional forms to pairwise interactions.
 
 
-def _get_species_parameters(params: Array, species: Array) -> Array:
+def _get_species_parameters(params: Parameter, species: Array
+                            ) -> Parameter:
   """Get parameters for interactions between species pairs."""
   # TODO(schsam): We should do better error checking here.
   if util.is_array(params):
@@ -183,28 +239,54 @@ def _get_species_parameters(params: Array, species: Array) -> Array:
     else:
       raise ValueError(
           'Params must be a scalar or a 2d array if using a species lookup.')
+  elif isinstance(params, ParameterTree):
+    p = params.tree
+    if params.mapping is ParameterTreeMapping.Global:
+      return p
+    elif params.mapping is ParameterTreeMapping.PerSpecies:
+      return tree_map(lambda x: x[species], p)
+    else:
+      raise ValueError('When species are present, ParameterTreeMapping must '
+                       f'be Global or PerSpecies. Found {params.mapping}.')
   return params
 
 
-def _get_matrix_parameters(params: Array, combinator: Callable) -> Array:
+def _get_matrix_parameters(params: Parameter,
+                           combinator: Callable[[Array, Array], Array]
+                           ) -> Parameter:
   """Get an NxN parameter matrix from per-particle parameters."""
   if util.is_array(params):
-    if len(params.shape) == 1:
+    if params.ndim == 1:
       return combinator(params[:, jnp.newaxis], params[jnp.newaxis, :])
-    elif len(params.shape) == 0 or len(params.shape) == 2:
+    elif params.ndim == 0 or params.ndim == 2:
       return params
     else:
-      raise NotImplementedError
+      raise ValueError('Without species information, parameters must be '
+                       'an array of dimension 0, 1, or 2. '
+                       f'Found {params.ndim}.')
+  elif isinstance(params, ParameterTree):
+    M = ParameterTreeMapping
+    if params.mapping in (M.Global, M.PerBond):
+      return params.tree
+    elif params.mapping is M.PerParticle:
+      return tree_map(lambda p: combinator(p[:, None, ...], p[None, :, ...]),
+                      params.tree)
+    else:
+      raise ValueError('Without species information, ParameterTreeMapping '
+                       'must be Global, PerBond, or PerParticle. '
+                       f'Found {params.mapping}.')
   elif(isinstance(params, int) or isinstance(params, float) or
        jnp.issubdtype(params, jnp.integer) or
        jnp.issubdtype(params, jnp.floating)):
     return params
   else:
-    raise NotImplementedError
+    raise ValueError('Without species information, params must eitehr be an '
+                     'array, a ParameterTree, or a float. '
+                     f'Found {type(params)}.')
 
 
 def _kwargs_to_parameters(species: Array,
-                          kwargs: Dict[str, Array],
+                          kwargs: Dict[str, Parameter],
                           combinators: Dict[str, Callable]
                           ) -> Dict[str, Array]:
   """Extract parameters from keyword arguments."""
@@ -232,9 +314,9 @@ def _diagonal_mask(X: Array) -> Array:
         ('Diagonal mask can only mask rank-2 or rank-3 tensors. '
          'Found {}.'.format(len(X.shape))))
   N = X.shape[0]
-  # NOTE(schsam): It seems potentially dangerous to set nans to 0 here. However,
-  # masking nans also doesn't seem to work. So it also seems necessary. At the
-  # very least we should do some @ErrorChecking.
+  # NOTE(schsam): It seems potentially dangerous to set nans to 0 here.
+  # However, masking nans also doesn't seem to work. So it also seems
+  # necessary. At the very least we should do some error checking.
   X = jnp.nan_to_num(X)
   mask = f32(1.0) - jnp.eye(N, dtype=X.dtype)
   if len(X.shape) == 3:
@@ -277,9 +359,9 @@ def pair(fn: Callable[..., Array],
 
   Args:
     fn: A function that takes an ndarray of pairwise distances or displacements
-      of shape `[n, m]` or `[n, m, d_in]` respectively as well as kwargs specifying
-      parameters for the function. fn returns an ndarray of evaluations of shape
-      `[n, m, d_out]`.
+      of shape `[n, m]` or `[n, m, d_in]` respectively as well as kwargs
+      specifying parameters for the function. fn returns an ndarray of
+      evaluations of shape `[n, m, d_out]`.
     metric: A function that takes two ndarray of positions of shape
       `[spatial_dimension]` and `[spatial_dimension]` respectively and returns
       an ndarray of distances or displacements of shape `[]` or `[d_in]`
@@ -292,52 +374,51 @@ def pair(fn: Callable[..., Array],
       `species` giving the maximum number of types of particles. Note: that
       dynamic species specification is less efficient, because we cannot
       specialize shape information.
-    reduce_axis: A list of axes to reduce over. This is supplied to `jnp.sum` and
-      so the same convention is used.
+    reduce_axis: A list of axes to reduce over. This is supplied to `jnp.sum`
+      and so the same convention is used.
     keepdims: A boolean specifying whether the empty dimensions should be kept
-      upon reduction. This is supplied to `jnp.sum` and so the same convention is
-      used.
+      upon reduction. This is supplied to `jnp.sum` and so the same convention
+      is used.
     ignore_unused_parameters: A boolean that denotes whether dynamically
       specified keyword arguments passed to the mapped function get ignored
       if they were not first specified as keyword arguments when calling
       `smap.pair(...)`.
     kwargs: Arguments providing parameters to the mapped function. In cases
-      where no species information is provided these should be either 
+      where no species information is provided these should be either
 
-        1) a scalar 
+        1) a scalar
         2) an ndarray of shape `[n]`
         3) an ndarray of shape `[n, n]`,
-        4) a binary function that determines how per-particle parameters are to be combined
-        5) a binary function as well as a default set of parameters as in 2)
-      
+        4) a ParameterTree containing a PyTree of parameters and a mapping. See
+           ParameterTree for details.
+        5) a binary function that determines how per-particle parameters are to
+           be combined
+        6) a binary function as well as a default set of parameters as in 2)
+           or 4).
+
       If unspecified then this is taken to be the average of the
       two per-particle parameters. If species information is provided then the
-      parameters should be specified as either 
-      
-        1) a scalar 
+      parameters should be specified as either
+
+        1) a scalar
         2) an ndarray of shape `[max_species, max_species]`
+        3) a ParameterTree containing a PyTree of parameters and a mapping. See
+           ParameterTree for details.
 
   Returns:
     A function fn_mapped.
 
-    If species is `None` or statically specified then `fn_mapped` takes as arguments
-    an ndarray of positions of shape `[n, spatial_dimension]`.
+    If species is `None` or statically specified then `fn_mapped` takes as
+    arguments an ndarray of positions of shape `[n, spatial_dimension]`.
 
     If species is dynamic then `fn_mapped` takes as input an ndarray of shape
-    `[n, spatial_dimension]`, an integer ndarray of species of shape `[n]`, and an
-    integer specifying the maximum species.
+    `[n, spatial_dimension]`, an integer ndarray of species of shape `[n]`, and
+    an integer specifying the maximum species.
 
     The mapped function can also optionally take keyword arguments that get
     threaded through the metric.
   """
 
-  # Each application of vmap adds a single batch dimension. For computations
-  # over all pairs of particles, we would like to promote the metric function
-  # from one that computes the displacement / distance between two vectors to
-  # one that acts over the cartesian product of two sets of vectors. This is
-  # equivalent to two applications of vmap adding one batch dimension for the
-  # first set and then one for the second.
-
   kwargs, param_combinators = _split_params_and_combinators(kwargs)
 
   merge_dicts = partial(util.merge_dicts,
@@ -408,16 +489,17 @@ def pair(fn: Callable[..., Array],
 
 def _get_neighborhood_matrix_params(format: partition.NeighborListFormat,
                                     idx: Array,
-                                    params: Array,
-                                    combinator: Callable) -> Array:
+                                    params: Parameter,
+                                    combinator: Callable[[Array, Array], Array]
+                                    ) -> Parameter:
   if util.is_array(params):
-    if len(params.shape) == 1:
+    if params.ndim == 1:
       if partition.is_sparse(format):
         return space.map_bond(combinator)(params[idx[0]], params[idx[1]])
       else:
         return combinator(params[:, None], params[idx])
         return space.map_neighbor(combinator)(params, params[idx])
-    elif len(params.shape) == 2:
+    elif params.ndim == 2:
       def query(id_a, id_b):
         return params[id_a, id_b]
       if partition.is_sparse(format):
@@ -425,28 +507,54 @@ def _get_neighborhood_matrix_params(format: partition.NeighborListFormat,
       else:
         query = vmap(vmap(query, (None, 0)))
         return query(jnp.arange(idx.shape[0], dtype=jnp.int32), idx)
-    elif len(params.shape) == 0:
+    elif params.ndim == 0:
       return params
     else:
-      raise NotImplementedError()
+      raise ValueError('Parameter array must be either a scalar, a vector, '
+                       f'or a matrix. Found ndim={params.ndim}.')
+  elif isinstance(params, ParameterTree):
+    if params.mapping is ParameterTreeMapping.Global:
+      return params.tree
+    elif params.mapping is ParameterTreeMapping.PerParticle:
+      if partition.is_sparse(format):
+        c_fn = space.map_bond(combinator)
+        return tree_map(lambda p: c_fn(p[idx[0]], p[idx[1]]), params.tree)
+      else:
+        c_fn = space.map_neighbor(combinator)
+        return tree_map(lambda p: c_fn(p, p[idx]), params.tree)
+    elif params.mapping is ParameterTreeMapping.PerBond:
+      def query(p, id_a, id_b):
+        return p[id_a, id_b]
+      if partition.is_sparse(format):
+        c_fn = lambda p: space.map_bond(partial(query, p))(idx[0], idx[1])
+        return tree_map(c_fn, params.tree)
+      else:
+        r = jnp.arange(idx.shape[0], dtype=jnp.int32)
+        c_fn = lambda p: vmap(vmap(partial(query, p), (None, 0)))(r, idx)
+        return tree_map(c_fn, params.tree)
+    else:
+      raise ValueError('Without species information ParameterTreeMapping '
+                       f'be Global or PerParticle. Found {params.mapping}.')
   elif(isinstance(params, int) or
        isinstance(params, float) or
        jnp.issubdtype(params, jnp.integer) or
        jnp.issubdtype(params, jnp.floating)):
     return params
   else:
-    raise NotImplementedError()
+    raise ValueError('Parameter must be an array, a ParameterTree, or a '
+                     f'float. Found {type(params)}.')
 
 def _get_neighborhood_species_params(format: partition.NeighborListFormat,
                                      idx: Array,
                                      species: Array,
-                                     params: Array) -> Array:
+                                     params: Parameter) -> Parameter:
   """Get parameters for interactions between species pairs."""
   # TODO(schsam): We should do better error checking here.
-  def lookup(species_a, species_b):
-    return params[species_a, species_b]
+  def lookup(p, species_a, species_b):
+    return p[species_a, species_b]
 
   if util.is_array(params):
+    lookup = partial(lookup, params)
     if len(params.shape) == 2:
       if partition.is_sparse(format):
         return space.map_bond(lookup)(species[idx[0]], species[idx[1]])
@@ -458,6 +566,21 @@ def _get_neighborhood_species_params(format: partition.NeighborListFormat,
     else:
       raise ValueError(
           'Params must be a scalar or a 2d array if using a species lookup.')
+  elif isinstance(params, ParameterTree):
+    if params.mapping is ParameterTreeMapping.Global:
+      return params.tree
+    elif params.mapping is ParameterTreeMapping.PerSpecies:
+      if partition.is_sparse(format):
+        l_fn = lambda p: space.map_bond(partial(lookup, p))(species[idx[0]],
+                                                            species[idx[1]])
+        return tree_map(l_fn, params.tree)
+      else:
+        l_fn = lambda p: vmap(vmap(partial(lookup, p), (None, 0)))(
+          species, species[idx])
+        return tree_map(l_fn, params.tree)
+    else:
+      raise ValueError('Parameter tree mapping must be either Global or '
+                       'PerSpecies if using a species lookup.')
   return params
 
 def _neighborhood_kwargs_to_params(format: partition.NeighborListFormat,
@@ -468,8 +591,7 @@ def _neighborhood_kwargs_to_params(format: partition.NeighborListFormat,
                                    ) -> Dict[str, Array]:
   out_dict = {}
   for k in kwargs:
-    if species is None or (
-        util.is_array(kwargs[k]) and kwargs[k].ndim == 1):
+    if species is None or (util.is_array(kwargs[k]) and kwargs[k].ndim == 1):
       combinator = combinators.get(k, lambda x, y: 0.5 * (x + y))
       out_dict[k] = _get_neighborhood_matrix_params(format,
                                                     idx,
@@ -500,9 +622,9 @@ def pair_neighbor_list(fn: Callable[..., Array],
 
   Args:
     fn: A function that takes an ndarray of pairwise distances or displacements
-      of shape `[n, m]` or `[n, m, d_in]` respectively as well as kwargs specifying
-      parameters for the function. fn returns an ndarray of evaluations of
-      shape `[n, m, d_out]`.
+      of shape `[n, m]` or `[n, m, d_in]` respectively as well as kwargs
+      specifying parameters for the function. fn returns an ndarray of
+      evaluations of shape `[n, m, d_out]`.
     metric: A function that takes two ndarray of positions of shape
       `[spatial_dimension]` and `[spatial_dimension]` respectively and returns
       an ndarray of distances or displacements of shape `[]` or `[d_in]`
@@ -524,23 +646,28 @@ def pair_neighbor_list(fn: Callable[..., Array],
       if they were not first specified as keyword arguments when calling
       `smap.pair_neighbor_list(...)`.
     kwargs: Arguments providing parameters to the mapped function. In cases
-      where no species information is provided these should be either 
+      where no species information is provided these should be either
 
-        1) a scalar 
+        1) a scalar
         2) an ndarray of shape `[n]`
         3) an ndarray of shape `[n, n]`,
-        4) a binary function that determines how per-particle parameters are to be combined
-      
+        4) a ParameterTree containing a PyTree of parameters and a mapping. See
+           ParameterTree for details.
+        5) a binary function that determines how per-particle parameters are
+           to be combined
+
       If unspecified then this is taken to be the average of the
       two per-particle parameters. If species information is provided then the
-      parameters should be specified as either 
-      
-        1) a scalar 
+      parameters should be specified as either
+
+        1) a scalar
         2) an ndarray of shape `[max_species, max_species]`
-      
+        3) a ParameterTree containing a PyTree of parameters and a mapping. See
+           ParameterTree for details.
+
   Returns:
-    A function `fn_mapped` that takes an ndarray of floats of shape `[N, d_in]` of
-    positions and and ndarray of integers of shape `[N, max_neighbors]`
+    A function `fn_mapped` that takes an ndarray of floats of shape `[N, d_in]`
+    of positions and and ndarray of integers of shape `[N, max_neighbors]`
     specifying neighbors.
   """
   kwargs, param_combinators = _split_params_and_combinators(kwargs)
@@ -613,18 +740,17 @@ def triplet(fn: Callable[..., Array],
 
   Many empirical potentials in jax_md include three-body angular terms (e.g.
   Stillinger Weber). This utility function simplifies the loss computation
-  in such cases by converting a function that takes in two pairwise displacements
-  or distances to one that only requires the system as input.
+  in such cases by converting a function that takes in two pairwise
+  displacements or distances to one that only requires the system as input.
 
   Args:
     fn: A function that takes an ndarray of two distances or displacements
-        from a central atom, both of shape `[n, m]` or `[n, m, d_in]` respectively,
-        as well as kwargs specifying parameters for the function.
+        from a central atom, both of shape `[n, m]` or `[n, m, d_in]`
+        respectively, as well as kwargs specifying parameters for the function.
     metric: A function that takes two ndarray of positions of shape
         `[spatial_dimensions]` and `[spatial_dimensions]` respectively and
-        returns an ndarray of distances or displacements of shape `[]` or `[d_in]`
-        respectively. The metric can optionally take a floating point time as a
-        third argument.
+        returns an ndarray of distances or displacements of shape `[]` or
+        `[d_in]` respectively.
     species: A list of species for the different particles. This should either
       be None (in which case it is assumed that all the particles have the same
       species), an integer ndarray of shape `[n]` with species data, or an
@@ -642,18 +768,19 @@ def triplet(fn: Callable[..., Array],
       if they were not first specified as keyword arguments when calling
       `smap.triplet(...)`.
     kwargs: Argument providing parameters to the mapped function. In cases
-        where no species information is provided, these should either be 
-          
+        where no species information is provided, these should either be
+
           1) a scalar
           2) an ndarray of shape `[n]` based on the central atom
           3) an ndarray of shape `[n, n, n]` defining triplet interactions.
 
         If species information is provided, then the parameters should
-        be specified as either 
-        
+        be specified as either
+
           1) a scalar
           2) an ndarray of shape `[max_species]`
-          3) an ndarray of shape `[max_species, max_species, max_species]` defining triplet interactions.
+          3) an ndarray of shape `[max_species, max_species, max_species]`
+             defining triplet interactions.
 
   Returns:
     A function `fn_mapped`.
@@ -668,6 +795,7 @@ def triplet(fn: Callable[..., Array],
     The mapped function can also optionally take keyword arguments that get
     threaded through the metric.
   """
+
   merge_dicts = partial(util.merge_dicts,
                         ignore_unused_parameters=ignore_unused_parameters)
 
